package org.waxworlds.importcontacts;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

public class Doit extends WizardActivity
{
	private final static int DIALOG_ERROR = 0;
	private final static int DIALOG_CONTINUEORABORT = 1;
	private final static int DIALOG_MERGEPROMPT = 2;

	private boolean _startedProgress;
	private int _maxProgress;
	private int _tmpProgress;
	private int _progress;
	protected String _dialogMessage;
	private Dialog _mergePromptDialog;
	private boolean _mergePromptAlwaysSelected;

	private int _countOverwrites;
	private int _countCreates;
	private int _countMerges;
	private int _countSkips;

	protected Importer _importer;

	public Handler _handler;

	public class DoitHandler extends Handler
	{
		@Override
		public void handleMessage( Message msg ) {
			switch( msg.what )
			{
			case Importer.MESSAGE_FINISHED:
				( (LinearLayout)findViewById( R.id.doit_closedisplay ) ).
						setVisibility( View.VISIBLE );
				break;
			case Importer.MESSAGE_FINISHED_BACK:
				( (Button)findViewById( R.id.back ) ).setEnabled( true );
				break;
			case Importer.MESSAGE_ERROR:
				Doit.this._dialogMessage = (String)msg.obj;
				showDialog( DIALOG_ERROR );
				break;
			case Importer.MESSAGE_CONTINUEORABORT:
				Doit.this._dialogMessage = (String)msg.obj;
				showDialog( DIALOG_CONTINUEORABORT );
				break;
			case Importer.MESSAGE_SETPROGRESSMESSAGE:
				( (TextView)findViewById( R.id.doit_percentage ) ).
						setText( (String)msg.obj );
				break;
			case Importer.MESSAGE_SETMAXPROGRESS:
				if( _maxProgress > 0 ) {
					if( _tmpProgress == _maxProgress )
						_tmpProgress = (Integer)msg.obj;
					if( _progress == _maxProgress )
						_progress = (Integer)msg.obj;
				}
				_maxProgress = (Integer)msg.obj;
				updateProgress();
				break;
			case Importer.MESSAGE_SETTMPPROGRESS:
				_tmpProgress = (Integer)msg.obj;
				updateProgress();
				break;
			case Importer.MESSAGE_SETPROGRESS:
				_startedProgress = true;
				_progress = (Integer)msg.obj;
				updateProgress();
				break;
			case Importer.MESSAGE_MERGEPROMPT:
				_dialogMessage = (String)msg.obj;
				showDialog( DIALOG_MERGEPROMPT );
				break;
			case Importer.MESSAGE_CONTACTOVERWRITTEN:
				_countOverwrites++;
				updateStats();
				break;
			case Importer.MESSAGE_CONTACTCREATED:
				_countCreates++;
				updateStats();
				break;
			case Importer.MESSAGE_CONTACTMERGED:
				_countMerges++;
				updateStats();
				break;
			case Importer.MESSAGE_CONTACTSKIPPED:
				_countSkips++;
				updateStats();
				break;
			default:
				super.handleMessage( msg );
			}
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		setContentView( R.layout.doit );
		super.onCreate( savedInstanceState );

		// hide page 2
		( findViewById( R.id.doit_page_2 ) ).setVisibility( View.GONE );

		// set up begin button
		Button begin = (Button)findViewById( R.id.doit_begin );
		begin.setOnClickListener( new View.OnClickListener() {
			public void onClick( View view ) {
				importContacts();
			}
		} );

		// set up close button
		Button close = (Button)findViewById( R.id.doit_close );
		close.setOnClickListener( new View.OnClickListener() {
			public void onClick( View view ) {
				setResult( RESULT_CANCELED );
				finish();
			}
		} );



		_startedProgress = false;
		_maxProgress = 0;
		_tmpProgress = 0;
		_progress = 0;
		_handler = new DoitHandler();

		_countOverwrites = 0;
		_countCreates = 0;
		_countMerges = 0;
		_countSkips = 0;

		updateProgress();
		updateStats();
	}

	@Override
	protected void onPause()
	{
		super.onPause();

		abortImport();

		// notify the user


	}

	@Override
	protected Dialog onCreateDialog(int id)
	{
		switch( id )
		{
		case DIALOG_ERROR:
			return new AlertDialog.Builder( this )
					.setIcon( R.drawable.alert_dialog_icon )
					.setTitle( R.string.error_title )
					.setMessage( "" )
					.setPositiveButton( R.string.error_ok,
							new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog,
								int whichButton) {
							Doit.this._importer.wake();
						}
					} )
					.setOnCancelListener( _dialogOnCancelListener )
					.create();
		case DIALOG_CONTINUEORABORT:
			return new AlertDialog.Builder( this )
					.setIcon( R.drawable.alert_dialog_icon )
					.setTitle( R.string.error_title )
					.setMessage( "" )
					.setPositiveButton( R.string.error_continue,
							new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog,
								int whichButton) {
							Doit.this._importer.wake(
									Importer.RESPONSE_POSITIVE );
						}
					} )
					.setNegativeButton( R.string.error_abort,
							new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog,
								int whichButton) {
							Doit.this._importer.wake(
									Importer.RESPONSE_NEGATIVE );
						}
					} )
					.setOnCancelListener( _dialogOnCancelListener )
					.create();
		case DIALOG_MERGEPROMPT:
			// custom layout in an AlertDialog
			LayoutInflater factory = LayoutInflater.from( this );
			final View dialogView = factory.inflate(
					R.layout.mergeprompt, null );
			( (CheckBox)dialogView.findViewById( R.id.mergeprompt_always ) ).
					setOnCheckedChangeListener(
							new CompoundButton.OnCheckedChangeListener() {
				public void onCheckedChanged( CompoundButton buttonView,
						boolean isChecked ) {
					Doit.this._mergePromptAlwaysSelected = isChecked;
				}
			} );
			( (Button)dialogView.findViewById( R.id.merge_keep ) ).
					setOnClickListener( _mergePromptButtonListener );
			( (Button)dialogView.findViewById( R.id.merge_overwrite ) ).
					setOnClickListener( _mergePromptButtonListener );
			( (Button)dialogView.findViewById( R.id.merge_merge ) ).
					setOnClickListener( _mergePromptButtonListener );
			_mergePromptAlwaysSelected = false;
			return new AlertDialog.Builder( this )
					.setIcon( R.drawable.alert_dialog_icon )
					.setTitle( R.string.mergeprompt_title )
					.setView( dialogView )
					.setOnCancelListener( _dialogOnCancelListener )
					.create();
		}
		return null;
	}

    private OnClickListener _mergePromptButtonListener = new OnClickListener() {
		public void onClick( View view ) {
			int responseExtra = _mergePromptAlwaysSelected?
					Importer.RESPONSEEXTRA_ALWAYS : Importer.RESPONSEEXTRA_NONE;
			Doit.this._mergePromptDialog.dismiss();
			Doit.this._mergePromptDialog = null;	// dont keep a reference!
			Doit.this._importer.wake( view.getId(), responseExtra );
		}
	};

	private DialogInterface.OnCancelListener _dialogOnCancelListener =
			new DialogInterface.OnCancelListener() {
		public void onCancel( DialogInterface dialog ) {
			setResult( RESULT_CANCELED );
			finish();
		}
	};

	@Override
	protected void onActivityResult( int requestCode, int resultCode,
			Intent data )
	{
		if( resultCode == RESULT_CANCELED )
			abortImport();
	}

	@Override
	protected void onPrepareDialog(int id, Dialog dialog)
	{
		switch( id )
		{
		case DIALOG_ERROR:	// fall through
		case DIALOG_CONTINUEORABORT:
			// set dialog message
			( (AlertDialog)dialog ).setMessage( _dialogMessage );
			break;
		case DIALOG_MERGEPROMPT:
			// set contact's name
			( (TextView)dialog.findViewById( R.id.mergeprompt_name ) ).setText(
					_dialogMessage );
			// and set up reference to dialog
			_mergePromptDialog = dialog;
			break;
		}

		super.onPrepareDialog( id, dialog );
	}

	private void importContacts()
	{
		// switch interfaces
		( findViewById( R.id.doit_page_1 ) ).setVisibility( View.GONE );
		( findViewById( R.id.doit_page_2 ) ).setVisibility( View.VISIBLE );

		// disable back button
		( (Button)findViewById( R.id.back ) ).setEnabled( false );

		// create importer
		_importer = new VCFImporter( this );

		// start the service's thread
		_importer.start();
	}

	private void updateProgress()
	{
		ProgressBar bar = (ProgressBar)findViewById( R.id.doit_progress );
		TextView outOf = (TextView)findViewById( R.id.doit_outof );

		if( _maxProgress > 0 )
		{
			bar.setMax( _maxProgress );
			bar.setSecondaryProgress( _tmpProgress );

			if( _startedProgress )
			{
				( (TextView)findViewById( R.id.doit_percentage ) ).setText(
						(int)Math.round( 100 * _progress / _maxProgress ) + "%" );
				outOf.setText( _progress + "/" + _maxProgress );
				bar.setProgress( _progress );
			}
		}
	}

	private void updateStats()
	{
		( (TextView)findViewById( R.id.doit_overwrites ) ).setText(
				"" + _countOverwrites );
		( (TextView)findViewById( R.id.doit_creates ) ).setText(
				"" + _countCreates );
		( (TextView)findViewById( R.id.doit_merges ) ).setText(
				"" + _countMerges );
		( (TextView)findViewById( R.id.doit_skips ) ).setText(
				"" + _countSkips );
	}

	private void abortImport()
	{
		if( _importer != null )
		{
			_importer.setAbort();
			while( true ) {
				try {
					_importer.join();
					break;
				}
				catch( InterruptedException e ) {}
			}
		}
	}
}
